import { isString } from 'gs-base';

function toCfgJson(data, space) {
  return JSON.stringify(data, (k, value) => {
    if (value instanceof Map) {
      return Object.fromEntries(value.entries());
    }
    if (value instanceof Set) {
      return Array.from(value);
    }
    if (value instanceof RegExp) {
      return value.toString();
    }
    return value;
  }, space);
}

var __defProp$3 = Object.defineProperty;
var __defNormalProp$3 = (obj, key, value) => key in obj ? __defProp$3(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$3 = (obj, key, value) => __defNormalProp$3(obj, typeof key !== "symbol" ? key + "" : key, value);
class ActionBuilder {
  constructor(page, actions) {
    __publicField$3(this, "actions");
    __publicField$3(this, "page");
    this.actions = actions;
    this.page = page;
  }
  get end() {
    return this.page;
  }
  date(...args) {
    return this.action("date", args);
  }
  dropdown(...args) {
    return this.action("dropdown", args);
  }
  radio(...args) {
    return this.action("radio", args);
  }
  step(...args) {
    return this.action("step", args);
  }
  text(...args) {
    return this.action("text", args);
  }
  click(...args) {
    return this.action("click", args);
  }
  fn(...args) {
    return this.action("fn", args);
  }
  key(...args) {
    this.actions.push(...args);
    return this;
  }
  action(type, args) {
    for (const arg of args) {
      if (isString(arg)) {
        this.actions.push({ type, cxt: arg });
      } else {
        arg.type = type;
        this.actions.push(arg);
      }
    }
    return this;
  }
}

var __defProp$2 = Object.defineProperty;
var __defNormalProp$2 = (obj, key, value) => key in obj ? __defProp$2(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$2 = (obj, key, value) => __defNormalProp$2(obj, key + "" , value);
class ChildBuilder extends ActionBuilder {
  constructor(page, children, childKey, title) {
    super(page, []);
    __publicField$2(this, "childRule");
    children.push(this.childRule = { title, childKey, childRules: this.actions });
  }
  query(queries) {
    this.childRule.queries = { ...this.childRule.queries, ...queries };
    return this;
  }
  options(options) {
    this.childRule.options = { ...this.childRule.options, ...options };
    return this;
  }
}

var __defProp$1 = Object.defineProperty;
var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$1 = (obj, key, value) => __defNormalProp$1(obj, typeof key !== "symbol" ? key + "" : key, value);
class CondFalseBuilder extends ActionBuilder {
  constructor(page, parent, actions) {
    super(page, actions);
    __publicField$1(this, "parent");
    this.parent = parent;
  }
  get back() {
    return this.parent;
  }
}
class CondBuilder extends CondFalseBuilder {
  constructor(page, parent, condRules, cxtKey, cond) {
    super(page, parent, []);
    __publicField$1(this, "condRule");
    condRules.push(this.condRule = { cond, cxtKey, trueRules: this.actions });
  }
  addFalseRules() {
    return new CondFalseBuilder(this.page, this.parent, this.actions);
  }
}

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, key + "" , value);
class GroupBuilder extends ActionBuilder {
  constructor(page, groups, groupKey, title) {
    super(page, []);
    __publicField(this, "groupRule");
    groups.push(this.groupRule = { title, groupKey, groupRules: this.actions });
  }
  query(queries) {
    this.groupRule.queries = { ...this.groupRule.queries, ...queries };
    return this;
  }
  options(options) {
    this.groupRule.options = { ...this.groupRule.options, ...options };
    return this;
  }
  addCondRules(cxtKey, cond) {
    return new CondBuilder(this.page, this, this.groupRule.condRules = [], cxtKey, cond);
  }
}

var __typeError = (msg) => {
  throw TypeError(msg);
};
var __accessCheck = (obj, member, msg) => member.has(obj) || __typeError("Cannot " + msg);
var __privateGet = (obj, member, getter) => (__accessCheck(obj, member, "read from private field"), getter ? getter.call(obj) : member.get(obj));
var __privateAdd = (obj, member, value) => member.has(obj) ? __typeError("Cannot add the same private member more than once") : member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
var __privateSet = (obj, member, value, setter) => (__accessCheck(obj, member, "write to private field"), member.set(obj, value), value);
var _rule;
function buildPage(url, queries) {
  return new PageRuleBuilder(url, queries);
}
class PageRuleBuilder {
  constructor(url, queries) {
    __privateAdd(this, _rule);
    __privateSet(this, _rule, {
      url,
      before: [],
      child: [],
      group: [],
      after: []
    });
    __privateGet(this, _rule).queries = queries || {};
  }
  query(queries) {
    __privateGet(this, _rule).queries = { ...__privateGet(this, _rule).queries, ...queries };
    return this;
  }
  options(options) {
    __privateGet(this, _rule).options = { ...__privateGet(this, _rule).options, ...options };
    return this;
  }
  addBefore() {
    return new ActionBuilder(this, __privateGet(this, _rule).before);
  }
  addChild(childKey, title) {
    return new ChildBuilder(this, __privateGet(this, _rule).child, childKey, title);
  }
  addGroup(groupKey, title) {
    return new GroupBuilder(this, __privateGet(this, _rule).group, groupKey, title);
  }
  addAfter() {
    return new ActionBuilder(this, __privateGet(this, _rule).after);
  }
  toJSON() {
    return __privateGet(this, _rule);
  }
  build() {
    return __privateGet(this, _rule);
  }
  toString() {
    return JSON.stringify(__privateGet(this, _rule), null, 2);
  }
}
_rule = new WeakMap();

const SetterConfig = {};
const DefaultSetter = "default";
function setSetterOption(parent, cfg) {
  if (SetterConfig[cfg.type]) {
    SetterConfig[cfg.type][parent] = cfg;
  } else {
    SetterConfig[cfg.type] = { [parent]: cfg };
  }
}
function getSetterOption(cfg, parent) {
  const map = SetterConfig[cfg.type];
  if (!map) {
    return cfg;
  }
  parent = cfg.parent || parent || DefaultSetter;
  const used = [];
  while (!used.includes(parent)) {
    used.push(parent);
    const pCfg = map[parent];
    parent = pCfg?.parent || DefaultSetter;
    if (pCfg) {
      cfg = { ...pCfg, ...cfg };
    }
  }
  return cfg;
}
function exportSetterOptions(space) {
  return toCfgJson(SetterConfig, space);
}
function importSetterOptions(json) {
  const data = JSON.parse(json);
  for (const values of Object.values(data)) {
    if (values)
      for (const [parent, config] of Object.entries(values)) {
        setSetterOption(parent, config);
      }
  }
}
function getSetterOptionType(wrap) {
  let str = "";
  for (const [type, options] of Object.entries(SetterConfig)) {
    const keys = Object.keys(options).filter((k) => k !== DefaultSetter);
    str += `${wrap ? "  " : ""}${type}: ${keys.map((k) => `'${k}'`).join("|")}
`;
  }
  if (wrap) {
    return `{
${str}}
`;
  }
  return str;
}

setSetterOption(DefaultSetter, {
  type: "text",
  inputQuery: "input:not([disabled]):not([readonly]):not([type=hidden]),textarea:not([disabled]):not([readonly])"
});

export { ActionBuilder, ChildBuilder, CondBuilder, CondFalseBuilder, DefaultSetter, GroupBuilder, PageRuleBuilder, buildPage, exportSetterOptions, getSetterOption, getSetterOptionType, importSetterOptions, setSetterOption, toCfgJson };
